import hashlib
from django import forms
from ccpages.models import Page


class PageAdminForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(PageAdminForm, self).__init__(*args, **kwargs)
        # TODO: is there a better way to prevent the TreeNodeChoiceField
        # form widget displaying invalid moves?
        if self.instance.pk is not None:
            self.fields['parent'].queryset = self.fields['parent'].queryset\
                    .exclude(
                            pk__in=[self.instance.pk]
                    ).exclude(
                            pk__in=[d.pk for d in self.instance.get_children()]
                    )

    class Meta:
        model = Page

    class Media:
        css = {
                'screen': ('ccpages/admin.css',)
            }

class PagePasswordForm(forms.Form):

    password = forms.CharField(widget=forms.PasswordInput())

    def __init__(self, *args, **kwargs):
        self.page = kwargs.pop('page', None)
        super(PagePasswordForm, self).__init__(*args, **kwargs)


    def clean_password(self):
        page_hash = self.page.hash
        password = self.cleaned_data['password']
        user_hash = hashlib.sha1(password).hexdigest()
        if user_hash == page_hash:
            self.hash = user_hash
            return password
        raise forms.ValidationError('Incorrect password')



from django import template
from django.conf import settings
from ccpages.models import Page

register = template.Library()

@register.assignment_tag
def ccpages_nav_breadcrumbs(page):
    """returns a breadcrumb of pages for a given page"""
    return Page.objects.nav_breadcrumbs(page)

@register.assignment_tag
def ccpages_nav_global():
    """returns the global pages"""
    return Page.objects.nav_global()

@register.assignment_tag
def ccpages_nav_local(page):
    """returns the pages for a given page's root"""
    return Page.objects.nav_local(page)

@register.filter
def icon_for_attachment(attachment, size="small"):
    """based on the file extention this will return a suitable icon"""
    path, ext = attachment.src.path.rsplit('.', 1)
    document_name = path.rsplit('/', 1)
    document_name.reverse()

    if ext == 'docx':
        ext = 'doc'

    extentions = ('aac','ai','aiff','avi','bmp','c','cpp','css',
    'dat','dmg','doc','dotx','dwg','dxf','eps','exe','flv','gif',
    'h','hpp','html','ics','iso','java','jpg','key','mid','mp3',
    'mp4','mpg','odf','ods','odt','otp','ots','ott','pdf','php',
    'ppt','psd','py','qt','rar','rb','rtf','sql','tga','tgz',
    'tiff','txt','wav','xls','xlsx','xml','yml','zip')

    if ext in extentions:
        image = '%s/%s.png' % (size, ext)
    else:
        image = '%s/_blank.png' % size

    return '%sccpages/img/icons/%s' % (
            settings.STATIC_URL,
            image)



from django.core.handlers.base import BaseHandler
from django.test.client import RequestFactory
from ccpages.models import Page



class MockRequest(RequestFactory):
    def request(self, **request):
        request = RequestFactory.request(self, **request)
        handler = BaseHandler()
        handler.load_middleware()
        for mw in handler._request_middleware:
            if mw(request):
                raise Exception('middleware returned a reponse')
        return request



from django.conf.urls import patterns, include, url


urlpatterns = patterns('',
    url(r'^(?P<slug>[\w\-]+)/password\.html$',
        'ccpages.views.password',
        name='password'),
    url(r'^(?P<slug>[\w\-]+)\.html$',
        'ccpages.views.view',
        name='view'),
    url(r'^$',
        'ccpages.views.index',
        name='index'),
)



from decimal import Decimal
from django.test import TestCase
from ccpages.forms import PagePasswordForm
from ccpages.models import Page


class ListenerTestCases(TestCase):

    def test_hash_if_password(self):
        """A hash is generated on save if page has password"""
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.password = 'ha'
        page1.status = Page.VISIBLE
        page1.save()
        # get the page
        p = Page.objects.get(pk=page1.pk)
        # we have a hash
        self.assertEqual(
                p.hash,
                'f9fc27b9374ad1e3bf34fdbcec3a4fd632427fed')
    
    def test_hash_if_no_password(self):
        """A hash is not generated on save if page has no password"""
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # get the page
        p = Page.objects.get(pk=page1.pk)
        # we have no hash
        self.assertFalse(p.hash)



from django.contrib import admin
from ccpages.models import Page, PageImage, PageAttachment
from ccpages.forms import PageAdminForm


class PageImageInline(admin.TabularInline):
    model = PageImage


class PageAttachmentInline(admin.TabularInline):
    model = PageAttachment


class PageAdmin(admin.ModelAdmin):
    form = PageAdminForm
    search_fields = (
            'title',
            'content',)
    list_filter = (
            'status',)
    list_display = (
            'title',
            'order',
            'status')
    list_editable = (
            'order',
            'status',)
    save_on_top = True
    inlines = [
            PageImageInline,
            PageAttachmentInline]
    prepopulated_fields = {'slug': ('title',)}
    fieldsets = (
        (None, {
            'fields': ('parent',
                        'status',
                        'title',
                        'content',),
            'classes': ('content',)
        }),
        ('Other Stuff', {
            'fields': ('slug',
                        'order',
                        'password',),
            'classes': ('collapse', 'other')
        })
    )


admin.site.register(Page, PageAdmin)



import hashlib

def create_hash(sender, instance, **kwargs):
    """If a page has a password then create a hash for it"""
    if instance.password is None:
        return None
    instance.hash = hashlib.sha1(instance.password).hexdigest()



from decimal import Decimal
from django.test import TestCase
from ccpages.models import Page


class ManagerTestCases(TestCase):

    def setUp(self):
        """setup"""

    def tearDown(self):
        """tear it all down"""

    def test_navbreadcrumbs(self):
        """The navbreadcrums method returns all visible decendants of
        a page all the way to the root"""
        # make 1 page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # make 2 page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.order = Decimal('2')
        page2.status = Page.VISIBLE
        page2.parent = page1
        page2.save()
        # make 3 page
        page3 = Page()
        page3.title = '3'
        page3.slug = '3'
        page3.content = '3'
        page3.order = Decimal('3')
        page3.status = Page.HIDDEN
        page3.parent = page1
        page3.save()
        # make 4 page
        page4 = Page()
        page4.title = '4'
        page4.slug = '4'
        page4.content = '4'
        page4.order = Decimal('4')
        page4.status = Page.VISIBLE
        page4.parent = page1
        page4.save()
        # make 5 page
        page5 = Page()
        page5.title = '5'
        page5.slug = '5'
        page5.content = '5'
        page5.order = Decimal('5')
        page5.status = Page.VISIBLE
        page5.save()
        # get the breadcrumb for page1
        p1_bc = Page.objects.nav_breadcrumbs(page1)
        # none because its a root
        self.assertEqual(0, p1_bc.count())
        # get page4
        p4_bc = Page.objects.nav_breadcrumbs(page4)
        self.assertEqual(1, p4_bc.count())
        # now make them all decendants of each other
        page3.parent = page2
        page3.save()
        page4.parent = page3
        page4.save()
        page5.parent = page4
        page5.save()
        # get page5
        p5_bc = Page.objects.nav_breadcrumbs(page5)
        # now only three in length because page 3 is hidden
        self.assertEqual(3, p5_bc.count())

    def test_navglobal(self):
        """the navglobal method returns all visible children of the
        root of a parent in the correct order"""
        # make 1 page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # make 2 page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.order = Decimal('2')
        page2.status = Page.VISIBLE
        page2.save()
        # make 3 page
        page3 = Page()
        page3.title = '3'
        page3.slug = '3'
        page3.content = '3'
        page3.order = Decimal('3')
        page3.status = Page.HIDDEN
        page3.save()
        # there will be two return by the manager methor
        nav = Page.objects.nav_global()
        self.assertEqual(2, nav.count())
        # make page 3 visible and now we have three
        page3.status = Page.VISIBLE
        page3.save()
        nav = Page.objects.nav_global()
        self.assertEqual(3, nav.count())


    def test_nav_local(self):
        """the nav_local method returns all visible children of the
        root of a parent in the correct order"""
        # make 1 page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # make 2 page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.order = Decimal('2')
        page2.status = Page.VISIBLE
        page2.parent = page1
        page2.save()
        # make 3 page
        page3 = Page()
        page3.title = '3'
        page3.slug = '3'
        page3.content = '3'
        page3.order = Decimal('3')
        page3.status = Page.HIDDEN
        page3.parent = page1
        page3.save()
        # make 4 page
        page4 = Page()
        page4.title = '4'
        page4.slug = '4'
        page4.content = '4'
        page4.order = Decimal('4')
        page4.status = Page.VISIBLE
        page4.parent = page1
        page4.save()
        # make 5 page
        page5 = Page()
        page5.title = '5'
        page5.slug = '5'
        page5.content = '5'
        page5.order = Decimal('5')
        page5.status = Page.VISIBLE
        page5.save()
        # the local nav for page 5 is empty
        p5_nav = Page.objects.nav_local(page5)
        self.assertEqual(0, p5_nav.count())
        # the local nav for p4 has 2 pages because one p3 is hidden
        p4_nav = Page.objects.nav_local(page4)
        self.assertEqual(2, p4_nav.count())
        # give page2 a lower order and now p4 is first
        page2.order = Decimal('100')
        page2.save()
        p2_nav = Page.objects.nav_local(page2)
        self.assertEqual(p2_nav[0].pk, page4.pk)
        self.assertEqual(p2_nav[1].pk, page2.pk)
        # make p4 orders waaay lower
        page4.order = Decimal('3000')
        page4.save()
        p4_nav = Page.objects.nav_local(page4)
        self.assertEqual(p4_nav[0].pk, page2.pk)
        self.assertEqual(p4_nav[1].pk, page4.pk)
    def test_visible(self):
        """The visible method returns only visible items"""
        # make a page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.HIDDEN
        page1.save()
        # make another page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.status = Page.VISIBLE
        page2.save()
        # the default page is the first one
        self.assertEqual(1, Page.custom.visible().count())
        # make the first one visible and we have a new default
        page1.status = Page.VISIBLE
        page1.save()
        self.assertEqual(2, Page.custom.visible().count())

    def test_default(self):
        """The default method of the custom manager returns the 
        instance with the highest order"""
        # make a page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.VISIBLE
        page1.save()
        # make another page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.status = Page.VISIBLE
        page2.save()
        # the default page is the first one
        default = Page.custom.default()
        self.assertEqual(default.pk, page1.pk)

    def test_default_page_must_be_visible(self):
        """The default page must be visible"""
        # make a page
        page1 = Page()
        page1.title = '1'
        page1.slug = '1'
        page1.content = '1'
        page1.order = Decimal('1')
        page1.status = Page.HIDDEN
        page1.save()
        # make another page
        page2 = Page()
        page2.title = '2'
        page2.slug = '2'
        page2.content = '2'
        page2.status = Page.VISIBLE
        page2.save()
        # the default page is the first one
        default = Page.custom.default()
        self.assertEqual(default.pk, page2.pk)
        # make the first one visible and we have a new default
        page1.status = Page.VISIBLE
        page1.save()
        default = Page.custom.default()
        self.assertEqual(default.pk, page1.pk)




import os
from distutils.core import setup
from ccpages import VERSION


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)

core_packages = [
    'ccpages',
]
for package in core_packages:
    for dirpath, dirnames, filenames in os.walk(package):
        # Ignore dirnames that start with '.'
        for i, dirname in enumerate(dirnames):
            if dirname.startswith('.'): del dirnames[i]
        if '__init__.py' in filenames:
            packages.append('.'.join(fullsplit(dirpath)))
        elif filenames:
            data_files.append(
                    [dirpath, [os.path.join(dirpath, f) for f in filenames]])


setup(
    name="django-ccpages",
    version=VERSION,
    license = 'BSD 3 Clause',
    description='A lightweight pages appliction for Django',
    author='c&c',
    author_email='studio@designcc.co.uk',
    url='https://github.com/designcc/django-ccpages',
    package_data = {
        'ccpages' : [
            'templates/ccpages/*.html',
        ],
    },
    packages=packages,
    data_files=data_files,
    install_requires = [
        'django-mptt',
        'django-ccthumbs',
        'markdown'])



from django.db import models


class PagesManager(models.Manager):

    def default(self):
        """returns the lowest ordered page as the default"""
        try:
            return super(PagesManager, self)\
                    .get_query_set()\
                    .filter(status=self.model.VISIBLE)\
                    .order_by('order')[0]
        except IndexError:
            raise self.model.DoesNotExist

    def visible(self):
        """returns the visible pages"""
        return super(PagesManager, self)\
                .get_query_set()\
                .filter(status=self.model.VISIBLE)

    def nav_breadcrumbs(self, page):
        """returns the hierarchy of pages from a page to the root"""
        return page\
                .get_ancestors()\
                .filter(status=self.model.VISIBLE)

    def nav_global(self):
        """returns the visible root pages for use in the global 
        navigation"""
        return super(PagesManager, self)\
                .get_query_set()\
                .filter(
                        status=self.model.VISIBLE,
                        parent=None)

    def nav_local(self, page):
        """returns the children of the root node of the supplied
        page for use in local navigation"""
        return page\
                .get_root()\
                .get_children()\
                .filter(status=self.model.VISIBLE)\
                .order_by('order')

